from blacs.tab_base_classes import MODE_MANUAL, MODE_TRANSITION_TO_BUFFERED, MODE_TRANSITION_TO_MANUAL, MODE_BUFFERED, define_state
from blacs.device_base_class import DeviceTab

class TimeBaseTab(DeviceTab):
    def initialise_GUI(self):  
        connection = self.settings['connection_table'].find_by_name(self.device_name)
        props = connection.properties
        self.hostname = props["hostname"]
        self.port = props["port"]
        # Capabilities
        self.base_units =    {'freq':'Hz',                 'amp':'dBm',}
        self.base_min =      {'freq':props["min_freq"],    'amp':props["min_amp"],}
        self.base_max =      {'freq':props["max_freq"],    'amp':props["max_amp"],}
        self.base_step =     {'freq':1e6,                  'amp':0.1,}
        self.base_decimals = {'freq':0,                    'amp':1,}
        
        # Create DDS Output objects
        dds_prop = {}
        channels = [ch.parent_port for ch in connection.child_list.values()]
        channels.sort() # Make sure they are shown in alphabetical order in BLACS
        for ch in channels:
            dds_prop[ch] = {"gate":{}}
            for subchnl in ['freq', 'amp']:
                dds_prop[ch][subchnl] = {'base_unit':self.base_units[subchnl], 
                                                'min':self.base_min[subchnl],
                                                'max':self.base_max[subchnl],
                                                'step':self.base_step[subchnl],
                                                'decimals':self.base_decimals[subchnl],
                                                }
        # Create the output objects    
        self.create_dds_outputs(dds_prop)
        # Create widgets for output objects
        dds_widgets,_,_ = self.auto_create_widgets()
        # and auto place the widgets in the UI
        if props["mock"]:
            self.auto_place_widgets(("MOCK: DDS Outputs",dds_widgets))
        else:
            self.auto_place_widgets(("DDS Outputs",dds_widgets))

        # Set the capabilities of this device
        self.supports_remote_value_check(True)
        self.supports_smart_programming(True)
        

        # Create and set the primary worker
        self.create_worker(
            "main_worker",
            "user_devices.TimeBaseAOMDriver.blacs_workers.TimeBaseWorker",
            {"hostname":self.hostname, "port":self.port, "channels":tuple(dds_prop.keys()), "mock":props["mock"]})
        self.primary_worker = "main_worker"


    # Redefine function, where we don't need to call the Worker,
    # because the TimeBase AOM driver has only static values progammed.
    @define_state(MODE_TRANSITION_TO_BUFFERED,False)
    def abort_transition_to_buffered(self,workers=None):
        self.mode = MODE_MANUAL
        self.program_device()

        
    @define_state(MODE_BUFFERED,False)
    def abort_buffered(self,notify_queue):
        notify_queue.put([self.device_name,'success'])
        self.mode = MODE_MANUAL
        self.program_device()
        
            
    @define_state(MODE_BUFFERED,False)
    def transition_to_manual(self,notify_queue,program=False):
        self.mode = MODE_TRANSITION_TO_MANUAL
        
        # Update the GUI with the final values of the run:
        for channel, value in self._final_values.items():
            if channel in self._AO:
                self._AO[channel].set_value(value,program=False)
            elif channel in self._DO:
                self._DO[channel].set_value(value,program=False)
            elif channel in self._image:
                self._image[channel].set_value(value,program=False)
            elif channel in self._DDS:
                self._DDS[channel].set_value(value,program=False)
        
        notify_queue.put([self.device_name,'success'])
        self.mode = MODE_MANUAL
        
        if program:
            self.program_device()
        else:
            self._last_programmed_values = self.get_front_panel_values()

    @define_state(MODE_MANUAL,True)
    def check_remote_values(self):
        # For the TimeBase AOM Driver sometimes when the values are read,
        # it happens that one number is wrong (maybe some bitnoise in the communication?)
        # If the nex function call returns again the right values, there is still the warning 
        # message visible in BLACS. I added here and additional "hide" command, such that the 
        # warning dissapears completely in that case, when there was once an error in reading 
        # the remote values.
        self._changed_widget.hide()
        return super().check_remote_values()
